@@ -10,12 +10,23 @@
 #__version__ = '1.0.4a 2021-06-11'# flush the file after each section
 #__version__ = '1.0.4 2021-06-11'# if file exist then rename the existing file
 #__version__ = '1.0.5 2021-06-14'# handling of different returned maps
-__version__ = '1.0.6 2021-06-19'# filename moved from instantiation to new method: start(), timestamp is int(nanoseconds)
+#__version__ = '1.0.6 2021-06-19'# filename moved from instantiation to new method: start(), timestamp is int(nanoseconds)
+#__version__ = '1.0.7 2021-06-20'# Docstrings updated
+#__version__ = '1.0.9 2021-06-21'# new keyword: self.use_single_float
+#__version__ = '1.0.10 2021-06-22'# separate events for exit and stop
+#__version__ = '1.0.11 2021-06-22'# intercept exception in _delivered()
+__version__ = '1.1.0 2021-06-23'# fixed bug when subscriptions was multiplied every start()
+
+#TODO: consider to replace msgpack_numpy with something simple and predictable.
+#The use_single_float has no efect in msgPack,
+#TODO: check how ints are handled: ideally they should be dynamically
+#converted to int8,int16,int32 or int64 depending of its value.
 
 import sys, time, string, copy
 import os, pathlib, datetime
 import threading
 import signal
+#from timeit import default_timer as timer
 
 import numpy as np
 import msgpack
@@ -26,10 +37,10 @@ msgpack_numpy.patch()
 SecDateTime, SecParagraph = 0,1
 Nano = 1000000000
 #````````````````````````````Helper functions`````````````````````````````````
-def printTime(): return time.strftime("%m%d:%H%M%S")
-def printi(msg): print(f'INFO_AS@{printTime()}: {msg}')
-def printw(msg): print(f'WARN_AS@{printTime()}: {msg}')
-def printe(msg): print(f'ERROR_AS@{printTime()}: {msg}')
+def _printTime(): return time.strftime("%m%d:%H%M%S")
+def _printi(msg): print(f'INFO_AS@{_printTime()}: {msg}')
+def _printw(msg): print(f'WARN_AS@{_printTime()}: {msg}')
+def _printe(msg): print(f'ERROR_AS@{_printTime()}: {msg}')
 
 def croppedText(txt, limit=200):
     if len(txt) > limit:
@@ -44,41 +55,65 @@ def shortkey(i:int):
     return s[i] if quotient==0 else s[quotient]+s[reminder]
 
 #````````````````````````````Serializer class`````````````````````````````````
-class apstrim ():
+class apstrim():
+    """Create the object streamer. 
+    publisher:      is a class, providing a subscribe() method,
+    devPar:         list of device:parameter strings,
+    sectionInterval: time between writing of the logBook sections
+    compression:    compression enable flag,
+    quiet:          do not print section writing progress,
+    use_single_float: Use single precision float type for float. (default: False)
+    eventExit:      is a threading.Event, which will be set when application
+     is about to exit.
+    """
     eventExit = threading.Event()
+    _eventStop = threading.Event()
 
     def __init__(self, publisher, devPars:list, sectionInterval=60.
-    , compression=False, quiet=False):
-        """Create the object streamer. 
-        publisher is a class, providing a subscribe() method,
-        devPar: list of device:parameter strings,
-        sectionInterval: time between writing of the logBook sections
-        compression: compression enable flag.
-        """
-        #printi(f'apstrim  {__version__}, sectionInterval {sectionInterval}')
+    , compress=False, quiet=False, use_single_float=True):
+        #_printi(f'apstrim  {__version__}, sectionInterval {sectionInterval}')
         self.publisher = publisher
         self.devPars = devPars
         self.sectionInterval = sectionInterval
         self.quiet = quiet
+        self.use_single_float = use_single_float
         signal.signal(signal.SIGINT, _safeExit)
         signal.signal(signal.SIGTERM, _safeExit)
-        self.headerSection = {'apstrim ':__version__
+
+        # create section Abstract
+        self.abstractSection = {'apstrim ':__version__
         , 'sectionInterval':sectionInterval}
-        if compression:
+        if compress:
             import lz4framed
             self.compress = lz4framed.compress
-            self.headerSection['compression'] = 'lz4framed'
+            self.abstractSection['compression'] = 'lz4framed'
         else:
             self.compress = None
-            self.headerSection['compression'] = 'None'
-        printi(f'Header section: {self.headerSection}')
+            self.abstractSection['compression'] = 'None'
+        _printi(f'Abstract section: {self.abstractSection}')
+
+        # create section Abbreviations
+        self.pars = {}
+        for i,pname in enumerate(self.devPars):
+            devPar = tuple(pname.rsplit(':',1))
+            if True:#try:
+                self.publisher.subscribe(self._delivered, devPar)
+            else:#except Exception as e:
+                __printe(f'Subscription failed for {pname}: {e}')
+                continue
+            self.pars[pname] = [shortkey(i)]
+        _printi(f'parameters: {self.pars}')
+        self.abbreviationSection = msgpack.packb({'parameters':self.pars}
+        , use_single_float=self.use_single_float)
+
         self.lock = threading.Lock()
 
-
-    def start(self, fileName:str):
-        """Start streaming of data objects to logbook file
+    def start(self, fileName='apstrim.aps'):
+        """Start streaming of the data objects to logbook file.
+        If file is already exist then it will be renamed and
+        a new file will be open with the provided name.
         """
-        # if file exists then rename the existing file
+        self._eventStop.clear()
         try:
             modificationTime = pathlib.Path(fileName).stat().st_mtime
             dt = datetime.datetime.fromtimestamp(modificationTime)
@@ -87,35 +122,28 @@ class apstrim ():
             except:    fname,ext = fileName,''
             otherName = fname + suffix + '.' + ext
             os.rename(fileName, otherName)
-            printw(f'Existing file {fileName} have been renamed to {otherName}')
+            _printw(f'Existing file {fileName} have been renamed to {otherName}')
         except Exception as e:
             pass
 
         self.logbook = open(fileName, 'wb')
-        self.logbook.write(msgpack.packb(self.headerSection))
-        printi(f'Logbook file: {fileName} created')
+
+        # write the sections Abstract and Abbreviations
+        self.logbook.write(msgpack.packb(self.abstractSection
+        , use_single_float=self.use_single_float))
+        self.logbook.write(self.abbreviationSection)
 
         self._create_logSection()
 
-        #printi('starting periodic thread')
+        #_printi('starting serialization  thread')
         myThread = threading.Thread(target=self._serialize_section)
         myThread.start()
 
-        self.pars = {}
-        for i,pname in enumerate(self.devPars):
-            devPar = tuple(pname.rsplit(':',1))
-            if True:#try:
-                self.publisher.subscribe(self._delivered, devPar)
-            else:#except Exception as e:
-                printe(f'Subscription failed for {pname}: {e}')
-                continue
-            self.pars[pname] = [shortkey(i)]
-        printi(f'parameters: {self.pars}')
-        self.logbook.write(msgpack.packb({'parameters':self.pars}))
+        _printi(f'Logbook file: {fileName} created')
 
     def stop(self):
-        """Stop the streaming"""
-        self.eventExit.set()
+        """Stop the streaming."""
+        self._eventStop.set()
         #self.logbook.close()
 
     def _delivered(self, *args):
@@ -152,8 +180,18 @@ class apstrim ():
                         timestamp = int(pars[par]['timestamp']*Nano)
                     skey = self.pars[devPar+':'+par][0]
             except Exception as e:
-                printw(f'exception in unpacking: {e}')
+                _printw(f'exception in unpacking: {e}')
                 continue
+
+            if self.use_single_float:
+                # Changing numpy float64 to float32 halves the data volume
+                #ts = timer()
+                try:    
+                    if value.dtype=='float64':
+                        value = value.astype('float32')
+                    #print(f'Numpy f64->f32 reduction time: {round(timer()-ts,6)}')
+                except:    pass
+
             if timestamp in timestampedMap:
                 timestampedMap[timestamp][skey] = value
             else:
@@ -161,7 +199,10 @@ class apstrim ():
         #TODO: timestampedMap may need sorting
         #print(f'timestampedMap: {timestampedMap}')
         with self.lock:
-            self.logParagraph.append(list(timestampedMap.items())[0])
+            try:
+                self.logParagraph.append(list(timestampedMap.items())[0])
+            except Exception as e:
+                _printe(f'Exception in _delivered for {args}: {e}')
         
     def _create_logSection(self):
       with self.lock:
@@ -170,20 +211,22 @@ class apstrim ():
         self.logSection = (key, self.logParagraph)
 
     def _serialize_section(self):
-        #printi('serialize_section started')
+        #_printi('serialize_section started')
         periodic_update = time.time()
         stat = [0, 0]
         try:
-          while not self.eventExit.is_set():
-            self.eventExit.wait(self.sectionInterval)
+          while not self._eventStop.is_set():
+            self._eventStop.wait(self.sectionInterval)
             if len(self.logSection[SecParagraph]) == 0:
                 continue
 
             #print(f'write section {self.logSection}')
-            packed = msgpack.packb(self.logSection)
+            packed = msgpack.packb(self.logSection
+            , use_single_float=self.use_single_float)
             if self.compress is not None:
                 compressed = self.compress(packed)
-                packed = msgpack.packb(compressed)
+                packed = msgpack.packb(compressed
+                , use_single_float=self.use_single_float)
             self.logbook.write(packed)
             self.logbook.flush()
 
@@ -203,6 +246,7 @@ class apstrim ():
                 
 def _safeExit(_signo, _stack_frame):#, self=None):
     print('safeExit')
+    apstrim._eventStop.set()
     apstrim.eventExit.set()
     
                 
